import { linter } from "@codemirror/lint";
import Markdoc from "@markdoc/markdoc";
const markdocLinterExtension = (markdocConfig, lintConfig = {}) => linter(
  (view) => {
    const doc = view.state.doc;
    const docPos = (line, char = 0) => {
      try {
        return doc.line(line + 1).from + char;
      } catch (err) {
        return doc.line(line).to + char;
      }
    };
    const ast = Markdoc.parse(doc.toString());
    const errors = Markdoc.validate(ast, markdocConfig);
    return errors.map((err) => {
      const { start, end } = err.location ?? {
        start: { line: 0 },
        end: { line: 1 }
      };
      return {
        from: docPos(start.line, start.character),
        to: docPos(end.line, end.character),
        // prettier-ignore
        severity: err.error.level === "critical" ? "error" : err.error.level === "debug" ? "hint" : err.error.level,
        message: err.error.message,
        actions: []
      };
    });
  },
  {
    tooltipFilter(diagnostics, state) {
      return diagnostics.map((i) => i);
    },
    ...lintConfig
  }
);
export {
  markdocLinterExtension
};
