import { html } from "@codemirror/legacy-modes/mode/xml";
const htmlMode = html;
function last(array) {
  return array.at(-1);
}
function tokenUntil(stream, state, untilRegExp) {
  if (stream.sol()) {
    let indent = 0;
    for (; indent < state.indent; indent++) {
      if (!stream.eat(/\s/))
        break;
    }
    if (indent)
      return null;
  }
  let oldString = stream.string;
  let match = untilRegExp.exec(oldString.substr(stream.pos));
  if (match) {
    stream.string = oldString.substr(0, stream.pos + match.index);
  }
  function hideFirstChars(stream2, n, inner) {
    stream2.lineStart += n;
    try {
      return inner();
    } finally {
      stream2.lineStart -= n;
    }
  }
  let result = hideFirstChars(stream, state.indent, function() {
    return htmlMode.token(stream, state.htmlState);
  });
  stream.string = oldString;
  return result;
}
const config = {
  indentUnit: 2
};
const markdoc = {
  startState() {
    return {
      states: [],
      indent: 0,
      quoteKind: null,
      htmlState: htmlMode.startState(0)
    };
  },
  token(stream, state) {
    let match;
    switch (last(state.states)) {
      case void 0:
        if (stream.match(/^{%-?\s*comment\s*-?%}/)) {
          state.indent += config.indentUnit;
          state.states.push("comment");
          return "comment";
        } else if (stream.match(/^{%-?\s*raw\s*-?%}/)) {
          state.indent += config.indentUnit;
          state.states.push("raw");
          return "keyword";
        } else if (stream.match(/^{{/)) {
          state.states.push("object");
          return "keyword";
        } else if (match = stream.match(/^{%-?\s*(\w*)/)) {
          if (!match || typeof match == "boolean")
            return;
          if (match[1] == "endcase")
            state.indent -= 2 * config.indentUnit;
          else if (/^end/.test(match[1]))
            state.indent -= config.indentUnit;
          else if (match[1] == "case")
            state.indent += 2 * config.indentUnit;
          else if (/^(if|unless|for|tablerow)$/.test(match[1]))
            state.indent += config.indentUnit;
          state.states.push("tag");
          return "keyword";
        }
        return tokenUntil(stream, state, /{{|{%/);
      case "object":
      case "tag": {
        match = stream.match(/^['"]/);
        if (match && typeof match != "boolean") {
          state.states.push("string");
          state.quoteKind = match[0];
          return "string";
        } else if (stream.match(last(state.states) == "object" ? /^}}/ : /^%}/)) {
          state.states.pop();
        } else {
          stream.next();
        }
        return "keyword";
      }
      case "comment":
        if (stream.match(/^.*?{%-?\s*\/comment\s*-?%}/)) {
          state.indent -= config.indentUnit;
          state.states.pop();
        } else {
          stream.skipToEnd();
        }
        return "comment";
      case "raw":
        if (stream.match(/^{%-?\s*endraw\s*-?%}/)) {
          state.indent -= config.indentUnit;
          state.states.pop();
          return "keyword";
        }
        return tokenUntil(stream, state, /{%-?\s*endraw\s*-?%}/);
      case "string":
        match = stream.match(/^.*?(["']|\\[\s\S])/);
        if (!match) {
          stream.skipToEnd();
        } else if (typeof match == "boolean")
          ;
        else if (match[1] == state.quoteKind) {
          state.quoteKind = null;
          state.states.pop();
        }
        return "string";
    }
  },
  indent: function(state, textAfter, line) {
    let indent = state.indent, top = last(state.states);
    if (top == "comment" && /^{%-?\s*endcomment\s*-?%}/.test(textAfter))
      indent -= config.indentUnit;
    else if (top == "raw" && /^{%-?\s*endraw\s*-?%}/.test(textAfter))
      indent -= config.indentUnit;
    else if (top == void 0 && /^{%-?\s*endcase/.test(textAfter))
      indent -= 2 * config.indentUnit;
    else if (top == void 0 && /^{%-?\s*(when|end|els)/.test(textAfter))
      indent -= config.indentUnit;
    return indent + htmlMode.indent(state.htmlState, textAfter, line);
  }
};
export {
  markdoc
};
